package com.boman.gen.mapper;

import java.util.List;
import com.boman.domain.GenTableColumn;
import org.apache.ibatis.annotations.Param;

/**
 * 业务字段 数据层
 * 
 * @author ruoyi
 */
public interface GenTableColumnMapper
{
    /**
     * 根据表名称查询列信息
     * 
     * @param tableName 表名称
     * @return 列信息
     */
    public List<GenTableColumn> selectDbTableColumnsByName(String tableName);

    /**
     * 查询业务字段列表
     * 
     * @param tableId 业务字段编号
     * @return 业务字段集合
     */
    public List<GenTableColumn> selectGenTableColumnListByTableId(Long tableId);

    /**
     * 查询字段列表
     * @param genTableColumn
     * @return
     */
    public List<GenTableColumn> selectGenTableColumnList(GenTableColumn genTableColumn);

    /**
     * 根据gen_table_column的id查询字段对象
     * @param columnId
     * @return
     */
    public GenTableColumn selectGenTableColumnByColumnId(Long columnId);

    /**
     * 查询业务字段列表
     *
     * @param tableIdList 业务字段编号List
     * @return 业务字段集合
     */
    List<GenTableColumn> listByTableIdList(@Param("tableIdList") List<Long> tableIdList);

    /**
     * 新增业务字段
     * 
     * @param genTableColumn 业务字段信息
     * @return 结果
     */
    public int insertGenTableColumn(GenTableColumn genTableColumn);

    /**
     * 修改业务字段
     * 
     * @param genTableColumn 业务字段信息
     * @return 结果
     */
    public int updateGenTableColumn(GenTableColumn genTableColumn);

    /**
     * 删除业务字段
     * 
     * @param genTableColumns 列数据
     * @return 结果
     */
    public int deleteGenTableColumns(List<GenTableColumn> genTableColumns);

    /**
     * 批量删除业务字段
     * 
     * @param ids 需要删除的数据ID 表id
     * @return 结果
     */
    public int deleteGenTableColumnByIds(Long[] ids);

    /**
     * 批量删除业务字段
     * @param ids 字段id
     * @return
     */
    public int deleteGenTableColumnByColumnIds(Long[] ids);



    /**校验column_name是否重复
    * @Description
    * @author tjf
    * @Date 2021/3/25
    */
    public GenTableColumn checkColumnNameUnique(GenTableColumn genTableColumn);
}