package com.boman.gen.service;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import com.boman.common.core.constant.UserConstants;
import com.boman.common.core.text.Convert;
import com.boman.common.core.utils.DateUtils;
import com.boman.common.core.web.domain.AjaxResult;
import com.boman.common.log.enums.BusinessType;
import com.boman.domain.RoleEnum;
import com.boman.domain.TableSql;

import com.boman.gen.api.RemoteGenTableService;
import com.boman.gen.mapper.GenTableRelationMapper;
import com.boman.system.api.RemoteSysMenuService;
import com.boman.system.api.domain.SysMenu;
import org.apache.commons.io.IOUtils;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.boman.common.core.constant.Constants;
import com.boman.common.core.constant.GenConstants;
import com.boman.common.core.exception.CustomException;
import com.boman.common.core.text.CharsetKit;
import com.boman.common.core.utils.SecurityUtils;
import com.boman.common.core.utils.StringUtils;
import com.boman.common.core.utils.file.FileUtils;
import com.boman.domain.GenTable;
import com.boman.domain.GenTableColumn;
import com.boman.gen.mapper.GenTableColumnMapper;
import com.boman.gen.mapper.GenTableMapper;
import com.boman.gen.util.GenUtils;
import com.boman.gen.util.VelocityInitializer;
import com.boman.gen.util.VelocityUtils;

/**
 * 业务 服务层实现
 *
 * @author ruoyi
 */
@Service
public class GenTableServiceImpl implements IGenTableService {
    private static final Logger log = LoggerFactory.getLogger(GenTableServiceImpl.class);

    @Autowired
    private GenTableMapper genTableMapper;

    @Autowired
    private GenTableColumnMapper genTableColumnMapper;

    @Autowired
    private RemoteSysMenuService remoteSysMenuService;

    @Autowired
    private TableSqlServiceImpl tableSqlService;

    @Autowired
    private IGenTableColumnService tableColumnService;

    @Autowired
    private GenTableRelationMapper genTableRelationMapper;

    @Autowired
    private RemoteGenTableService remoteGenTableService;

    /**
     * 查询业务信息
     *
     * @param id 业务ID
     * @return 业务信息
     */
    @Override
    public GenTable selectGenTableById(Long id) {
        GenTable genTable = genTableMapper.selectGenTableById(id);
        Long tablePrimaryKey = genTable.getTablePrimaryKey();
        Long akColumn = genTable.getAkColumn();
        Long dkColumn = genTable.getDkColumn();
        List<GenTableColumn> columns = genTable.getColumns();
        int index = 0;
        for (GenTableColumn column : columns) {
            Long columnId = column.getColumnId();
            if (tablePrimaryKey != null && tablePrimaryKey.equals(columnId)) {
                genTable.setTablePrimaryKeyName(column.getColumnName());
                index++;
            }  if (akColumn != null && akColumn.equals(columnId)) {
                genTable.setAkColumnName(column.getColumnName());
                index++;
            }  if (dkColumn != null && dkColumn.equals(columnId)) {
                genTable.setDkColumnName(column.getColumnName());
                index++;
            }
            if (index == 3) {
                break;
            }
        }
        setTableFromOptions(genTable);
        return genTable;
    }

    @Override
    public GenTable getByTableId(Long id) {
        return genTableMapper.selectGenTableById(id);
    }

    /**
     * 查询业务列表
     *
     * @param genTable 业务信息
     * @return 业务集合
     */
    @Override
    public List<GenTable> selectGenTableList(GenTable genTable) {
        List<GenTable> genTables = genTableMapper.selectGenTableList(genTable);
        return genTables;
    }

    /**
     * 查询据库列表
     *
     * @param genTable 业务信息
     * @return 数据库表集合
     */
    @Override
    public List<GenTable> selectDbTableList(GenTable genTable) {
        return genTableMapper.selectDbTableList(genTable);
    }

    /**
     * 查询据库列表
     *
     * @param tableNames 表名称组
     * @return 数据库表集合
     */
    @Override
    public List<GenTable> selectDbTableListByNames(String[] tableNames) {
        return genTableMapper.selectDbTableListByNames(tableNames);
    }

    /**
     * 查询表名称业务信息
     *
     * @param tableName 表名称
     * @return 业务信息
     */
    @Override
    public GenTable selectGenTableByName(String tableName) {
        return genTableMapper.selectGenTableByName(tableName);
    }

    /**
     * 查询所有表信息
     *
     * @return 表信息集合
     */
    @Override
    public List<GenTable> selectGenTableAll() {
        return genTableMapper.selectGenTableAll();
    }

    /**
     * 修改业务
     *
     * @param genTable 业务信息
     * @return 结果
     */
    @Override
    @Transactional
    public void updateGenTable(GenTable genTable) {
        String options = JSON.toJSONString(genTable.getParams());
        genTable.setOptions(options);
        isMenu(genTable, BusinessType.UPDATE);
        int row = genTableMapper.updateGenTable(genTable);
        if (row > 0) {
            for (GenTableColumn cenTableColumn : genTable.getColumns()) {
                tableColumnService.updateGenTableColumn(cenTableColumn);
            }
        }
        remoteGenTableService.loadTable(new GenTable());
    }

    /**
     * 删除业务对象
     *
     * @param tableIds 需要删除的数据ID
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteGenTableByIds(Long[] tableIds) {
        genTableMapper.deleteGenTableByIds(tableIds);
        genTableColumnMapper.deleteGenTableColumnByIds(tableIds);
        tableSqlService.deleteTableSqlByTableIds(tableIds);
        //删除关联关系
        genTableRelationMapper.deleteGenTableRelationByParentId(tableIds);
        remoteGenTableService.loadTable(new GenTable());
    }

    /**
     * 导入表结构
     *
     * @param tableList 导入表列表
     */
    @Override
    @Transactional
    public void importGenTable(List<GenTable> tableList) {
        String operName = SecurityUtils.getUsername();
        try {
            for (GenTable table : tableList) {
                String tableName = table.getTableName();
                GenUtils.initTable(table, operName);
                int row = genTableMapper.insertGenTable(table);
                if (row > 0) {
                    // 保存列信息
                    List<GenTableColumn> genTableColumns = genTableColumnMapper.selectDbTableColumnsByName(tableName);
                    for (GenTableColumn column : genTableColumns) {
                        GenUtils.initColumnField(column, table);
                        genTableColumnMapper.insertGenTableColumn(column);
                    }
                }
            }
        } catch (Exception e) {
            throw new CustomException("导入失败：" + e.getMessage());
        }
    }

    /**
     * 预览代码
     *
     * @param tableId 表编号
     * @return 预览数据列表
     */
    @Override
    public Map<String, String> previewCode(Long tableId) {
        Map<String, String> dataMap = new LinkedHashMap<>();
        // 查询表信息
        GenTable table = genTableMapper.selectGenTableById(tableId);
        // 设置主子表信息
        setSubTable(table);
        // 设置主键列信息
        setPkColumn(table);
        VelocityInitializer.initVelocity();

        VelocityContext context = VelocityUtils.prepareContext(table);

        // 获取模板列表
        List<String> templates = VelocityUtils.getTemplateList(table.getTplCategory());
        for (String template : templates) {
            // 渲染模板
            StringWriter sw = new StringWriter();
            Template tpl = Velocity.getTemplate(template, Constants.UTF8);
            tpl.merge(context, sw);
            dataMap.put(template, sw.toString());
        }
        return dataMap;
    }

    /**
     * 生成代码（下载方式）
     *
     * @param tableName 表名称
     * @return 数据
     */
    @Override
    public byte[] downloadCode(String tableName) {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ZipOutputStream zip = new ZipOutputStream(outputStream);
        generatorCode(tableName, zip);
        IOUtils.closeQuietly(zip);
        return outputStream.toByteArray();
    }

    /**
     * 生成代码（自定义路径）
     *
     * @param tableName 表名称
     */
    @Override
    public void generatorCode(String tableName) {
        // 查询表信息
        GenTable table = genTableMapper.selectGenTableByName(tableName);
        // 设置主子表信息
        setSubTable(table);
        // 设置主键列信息
        setPkColumn(table);

        VelocityInitializer.initVelocity();

        VelocityContext context = VelocityUtils.prepareContext(table);

        // 获取模板列表
        List<String> templates = VelocityUtils.getTemplateList(table.getTplCategory());
        for (String template : templates) {
            if (!StringUtils.containsAny(template, "sql.vm", "api.js.vm", "index.vue.vm", "index-tree.vue.vm")) {
                // 渲染模板
                StringWriter sw = new StringWriter();
                Template tpl = Velocity.getTemplate(template, Constants.UTF8);
                tpl.merge(context, sw);
                try {
                    String path = getGenPath(table, template);
                    FileUtils.writeStringToFile(new File(path), sw.toString(), CharsetKit.UTF_8);
                } catch (IOException e) {
                    throw new CustomException("渲染模板失败，表名：" + table.getTableName());
                }
            }
        }
    }

    /**
     * 同步数据库
     *
     * @param tableName 表名称
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void synchDb(String tableName) {
        GenTable table = genTableMapper.selectGenTableByName(tableName);
        //获取表字段集合
        List<GenTableColumn> tableColumns = table.getColumns();
        //获取列名称集合
        List<String> tableColumnNames = tableColumns.stream().map(GenTableColumn::getColumnName).collect(Collectors.toList());
        //根据表名称查询原列信息
        List<GenTableColumn> dbTableColumns = genTableColumnMapper.selectDbTableColumnsByName(tableName);
        if (StringUtils.isEmpty(dbTableColumns)) {
            throw new CustomException("同步数据失败，原表结构不存在");
        }
        //获取原表列名称集合
        List<String> dbTableColumnNames = dbTableColumns.stream().map(GenTableColumn::getColumnName).collect(Collectors.toList());

        dbTableColumns.forEach(column -> {
            if (!tableColumnNames.contains(column.getColumnName())) {
                GenUtils.initColumnField(column, table);
                genTableColumnMapper.insertGenTableColumn(column);
            }
        });

        List<GenTableColumn> delColumns = tableColumns.stream().filter(column -> !dbTableColumnNames.contains(column.getColumnName())).collect(Collectors.toList());
        if (StringUtils.isNotEmpty(delColumns)) {
            genTableColumnMapper.deleteGenTableColumns(delColumns);
        }
    }

    /**
     * 批量生成代码（下载方式）
     *
     * @param tableNames 表数组
     * @return 数据
     */
    @Override
    public byte[] downloadCode(String[] tableNames) {
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ZipOutputStream zip = new ZipOutputStream(outputStream);
        for (String tableName : tableNames) {
            generatorCode(tableName, zip);
        }
        IOUtils.closeQuietly(zip);
        return outputStream.toByteArray();
    }

    /**
     * 查询表信息并生成代码
     */
    private void generatorCode(String tableName, ZipOutputStream zip) {
        // 查询表信息
        GenTable table = genTableMapper.selectGenTableByName(tableName);
        // 设置主子表信息
        setSubTable(table);
        // 设置主键列信息
        setPkColumn(table);

        VelocityInitializer.initVelocity();

        VelocityContext context = VelocityUtils.prepareContext(table);

        // 获取模板列表
        List<String> templates = VelocityUtils.getTemplateList(table.getTplCategory());
        for (String template : templates) {
            // 渲染模板
            StringWriter sw = new StringWriter();
            Template tpl = Velocity.getTemplate(template, Constants.UTF8);
            tpl.merge(context, sw);
            try {
                // 添加到zip
                zip.putNextEntry(new ZipEntry(VelocityUtils.getFileName(template, table)));
                IOUtils.write(sw.toString(), zip, Constants.UTF8);
                IOUtils.closeQuietly(sw);
                zip.flush();
                zip.closeEntry();
            } catch (IOException e) {
                log.error("渲染模板失败，表名：" + table.getTableName(), e);
            }
        }
    }

    /**
     * 修改保存参数校验
     *
     * @param genTable 业务信息
     */
    @Override
    public void validateEdit(GenTable genTable) {
        if (GenConstants.TPL_TREE.equals(genTable.getTplCategory())) {
            String options = JSON.toJSONString(genTable.getParams());
            JSONObject paramsObj = JSONObject.parseObject(options);
            if (StringUtils.isEmpty(paramsObj.getString(GenConstants.TREE_CODE))) {
                throw new CustomException("树编码字段不能为空");
            } else if (StringUtils.isEmpty(paramsObj.getString(GenConstants.TREE_PARENT_CODE))) {
                throw new CustomException("树父编码字段不能为空");
            } else if (StringUtils.isEmpty(paramsObj.getString(GenConstants.TREE_NAME))) {
                throw new CustomException("树名称字段不能为空");
            } else if (GenConstants.TPL_SUB.equals(genTable.getTplCategory())) {
                if (StringUtils.isEmpty(genTable.getSubTableName())) {
                    throw new CustomException("关联子表的表名不能为空");
                } else if (StringUtils.isEmpty(genTable.getSubTableFkName())) {
                    throw new CustomException("子表关联的外键名不能为空");
                }
            }
        }
    }

    /**
     * 代码生成业务表新增
     * 注意解决事务问题
     *
     * @param genTable
     * @return
     */
    @Override
    public AjaxResult insertGenTable(GenTable genTable) {
        //新增表成功的时候,新增字段公共字段
        int i = genTableMapper.insertGenTable(genTable);
        if (i > 0) {
            Long id = insertCommonColumn(genTable.getTableId());
            if (id != -1) {
                genTable.setTablePrimaryKey(id);
                genTableMapper.updateGenTable(genTable);
            }
            //判断是否是菜单
            isMenu(genTable, BusinessType.INSERT);
            //新建表时新增建表语句
            TableSql tableSql = new TableSql();
            tableSql.setTableId(genTable.getTableId());
            tableSqlService.insertTableSql(tableSql);
            remoteGenTableService.loadTable(new GenTable());
            return AjaxResult.success();
        }
        return AjaxResult.error();
    }

    /**
     * 新建表时新增对应公共字段
     *
     * @param tableId
     */
    public Long insertCommonColumn(Long tableId) {
        int num = 1000;
        GenTableColumn genTableColumnLog = new GenTableColumn();
        GenTableColumn genTableColumnBaseInfo = new GenTableColumn();

        genTableColumnLog.setTableId(tableId);
        genTableColumnLog.setColumnName("log_info");
        genTableColumnLog.setColumnComment("日志信息");
        genTableColumnLog.setColumnType("varchar(255)");
        genTableColumnLog.setIsPk("0");
        genTableColumnLog.setIsIncrement("0");
        genTableColumnLog.setIsInsert("1");
        genTableColumnLog.setQueryType("LIKE");
        genTableColumnLog.setHtmlType("input");
        genTableColumnLog.setSort(num);
        genTableColumnLog.setCreateBy("admin");
        genTableColumnLog.setHtmlType("HR");
        genTableColumnLog.setMask("000000");
        genTableColumnLog.setCreateTime(DateUtils.getNowDate());
        genTableColumnMapper.insertGenTableColumn(genTableColumnLog);

        genTableColumnBaseInfo.setTableId(tableId);
        genTableColumnBaseInfo.setColumnName("base_info");
        genTableColumnBaseInfo.setColumnComment("基本信息");
        genTableColumnBaseInfo.setColumnType("varchar(255)");
        genTableColumnBaseInfo.setIsPk("0");
        genTableColumnBaseInfo.setIsIncrement("0");
        genTableColumnBaseInfo.setIsInsert("1");
        genTableColumnBaseInfo.setQueryType("LIKE");
        genTableColumnBaseInfo.setHtmlType("input");
        genTableColumnBaseInfo.setSort(20);
        genTableColumnBaseInfo.setCreateBy("admin");
        genTableColumnBaseInfo.setHtmlType("HR");
        genTableColumnBaseInfo.setMask("000000");
        genTableColumnBaseInfo.setCreateTime(DateUtils.getNowDate());
        genTableColumnMapper.insertGenTableColumn(genTableColumnBaseInfo);


        Long columnId = -1L;
        String[] columnName = new String[]{"id", "create_by", "create_time", "update_by", "update_time", "is_del"};
        String[] columnComment = new String[]{"编号", "创建者", "创建时间", "更新者", "更新时间", "是否删除"};
        String[] columnType = new String[]{"bigint(20)", "varchar(64)", "datetime", "varchar(64)", "datetime", "char(1)"};
        String[] htmlType = new String[]{"input", "input", "datetime", "input", "datetime", "select"};
        for (int i = 0; i < columnName.length; i++) {
            GenTableColumn genTableColumn = new GenTableColumn();
            genTableColumn.setTableId(tableId);
            genTableColumn.setColumnName(columnName[i]);
            genTableColumn.setColumnComment(columnComment[i]);
            genTableColumn.setColumnType(columnType[i]);
            genTableColumn.setIsPk("0");
            genTableColumn.setIsInsert("1");
            genTableColumn.setQueryType("EQ");
            genTableColumn.setHtmlType(htmlType[i]);
            genTableColumn.setSort(num + i);
            genTableColumn.setCreateBy("admin");
            genTableColumn.setCreateTime(DateUtils.getNowDate());
            genTableColumn.setHrParentId(genTableColumnBaseInfo.getColumnId());
            genTableColumn.setMask("000000");
            if (i == 0) {
                genTableColumn.setSort(1);
                genTableColumn.setIsPk("1");
                genTableColumn.setHrParentId(genTableColumnLog.getColumnId());
            }
            genTableColumnMapper.insertGenTableColumn(genTableColumn);
            //获得id字段的id
            if (i == 0) {
                columnId = genTableColumn.getColumnId();
            }
        }
        return columnId;
    }

    /**
     * 判断是否是菜单
     *
     * @param genTable
     */
    public void isMenu(GenTable genTable, BusinessType type) {

        if (type.equals(BusinessType.UPDATE)) {
            //先查询之前是否是菜单
            GenTable genTableOld = genTableMapper.selectGenTableById(genTable.getTableId());
            if (UserConstants.ISMENU.equals(genTableOld.getIsMenu())) {
                return;
            }
        }
        String isMenu = genTable.getIsMenu();
        if (StringUtils.isNotBlank(isMenu) && isMenu.equals(UserConstants.ISMENU)) {
            //生成主菜单
            SysMenu sysMenu = new SysMenu();
            //表描述当菜单名称
            sysMenu.setMenuName(genTable.getTableComment());
            sysMenu.setParentId(2007L);
            //路由地址(使用表名称)
            sysMenu.setPath(genTable.getTableName());
            //组件
            sysMenu.setComponent("system/surface/index");
            sysMenu.setMenuType("C");
            sysMenu.setOrderNum("0");
            sysMenu.setIcon("system");
            sysMenu.setCreateBy(SecurityUtils.getUsername());
            sysMenu.setSysTableName(genTable.getTableName());
            sysMenu.setStatus("1");
            AjaxResult add = remoteSysMenuService.add(sysMenu);
            Long parentId = 0L;
            if (add.size() > 2) {
                String id = String.valueOf(add.get("data")) ;
                parentId = Long.valueOf(id);
            }
            //获取菜单id
            String menuRole = genTable.getMenuRole();
            String[] strings = Convert.toStrArray("",menuRole);
            if (strings.length > 0) {
                for (int i = 0; i < strings.length; i++) {
                    SysMenu menu = new SysMenu();
                    menu.setParentId(parentId);
                    menu.setMenuName(RoleEnum.getValue(strings[i]));
                    menu.setOrderNum(String.valueOf(i + 1));
                    menu.setMenuType("F");
                    menu.setIcon("#");
                    //路由
                    menu.setPath("#");
                    sysMenu.setCreateBy(SecurityUtils.getUsername());
                    remoteSysMenuService.add(menu);

                }
            }
        }
    }

    /**
     * 设置主键列信息
     *
     * @param table 业务表信息
     */
    public void setPkColumn(GenTable table) {
        for (GenTableColumn column : table.getColumns()) {
            if (column.getIsPk().equals(UserConstants.INCREMENT)) {
                table.setPkColumn(column);
                break;
            }
        }
        if (StringUtils.isNull(table.getPkColumn())) {
            table.setPkColumn(table.getColumns().get(0));
        }
        if (GenConstants.TPL_SUB.equals(table.getTplCategory())) {
            for (GenTableColumn column : table.getSubTable().getColumns()) {
                if (column.getIsPk().equals(UserConstants.INCREMENT)) {
                    table.getSubTable().setPkColumn(column);
                    break;
                }
            }
            if (StringUtils.isNull(table.getSubTable().getPkColumn())) {
                table.getSubTable().setPkColumn(table.getSubTable().getColumns().get(0));
            }
        }
    }

    /**
     * 设置主子表信息
     *
     * @param table 业务表信息
     */
    public void setSubTable(GenTable table) {
        String subTableName = table.getSubTableName();
        if (StringUtils.isNotEmpty(subTableName)) {
            table.setSubTable(genTableMapper.selectGenTableByName(subTableName));
        }
    }

    /**
     * 设置代码生成其他选项值
     *
     * @param genTable 设置后的生成对象
     */
    public void setTableFromOptions(GenTable genTable) {
        JSONObject paramsObj = JSONObject.parseObject(genTable.getOptions());
        if (StringUtils.isNotNull(paramsObj)) {
            String treeCode = paramsObj.getString(GenConstants.TREE_CODE);
            String treeParentCode = paramsObj.getString(GenConstants.TREE_PARENT_CODE);
            String treeName = paramsObj.getString(GenConstants.TREE_NAME);
            String parentMenuId = paramsObj.getString(GenConstants.PARENT_MENU_ID);
            String parentMenuName = paramsObj.getString(GenConstants.PARENT_MENU_NAME);

            genTable.setTreeCode(treeCode);
            genTable.setTreeParentCode(treeParentCode);
            genTable.setTreeName(treeName);
            genTable.setParentMenuId(parentMenuId);
            genTable.setParentMenuName(parentMenuName);
        }
    }

    /**
     * 获取代码生成地址
     *
     * @param table    业务表信息
     * @param template 模板文件路径
     * @return 生成地址
     */
    public static String getGenPath(GenTable table, String template) {
        String genPath = table.getGenPath();
        if (StringUtils.equals(genPath, "/")) {
            return System.getProperty("user.dir") + File.separator + "src" + File.separator + VelocityUtils.getFileName(template, table);
        }
        return genPath + File.separator + VelocityUtils.getFileName(template, table);
    }

    /**
     * 校验表名称是否存在
     *
     * @param genTable
     * @return
     */
    @Override
    public String checkTableNameUnique(GenTable genTable) {
        List<GenTable> genTables = genTableMapper.selectGenTableList(genTable);
        if (genTables.size() > 0) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }
}
